function [u,y_b,int]= controller(Kp,Ti,Td,y,int,y_b,yref)
%%
%This function simulates a standard PID-controller
%
% Variables: 
%               Output:
%               u:          controller signal
%               y_b:        Processoutput last sample
%               int:        Integral
%
%               Input:
%               Kp:         Proportional gain
%               Ti:         Integral time
%               Td:         Derivate time
%               y:          Process signal
%               int:        Integral
%               y_b:        Processoutput last sample
%
%               yref:       Reference
%
%Made by Bachelorgroup E1223 from Hgskolen i Sr-Trndelag on behalf of
%Siemens Trondheim.
%
%Project group:
%                   Anders Helle Mork
%                   Geir Tvetene Kristiansen
%                   Knut Toset
%                   Eskil hove Meringdal
%
%Contact at Siemens:
%                   Selvanathan Sivalingam

%% Controller settings
if Ti==0
    I=0;
    Ti=1;
else
    I=1;
end
r=yref;

%% Calculate Controll signal
e=r-y;

int=int+I*(Kp/Ti)*e;
der=Kp*Td*(y-y_b);
prop=Kp*e;

u=prop+(I*int)+der;

y_b=y;

%% Check limits (0-1)
if u < 0
    u=0;
end
if u > 1
    u=1;
end

return
end