function [dm_dt,P_wh_noise,h,w_liq_out,w_gas_out, w_total, P_all, w_out_mix, P_rt] = pipeline_riser(t,m,par,Z1,Z3,d_1)

m_liq_w=m(1,:);
m_gas_w=m(2,:);
m_liq_p=m(3,:);
m_gas_p=m(4,:);
m_liq_r=m(5,:);
m_gas_r=m(6,:);
%% %%%%%%%%%%%%%%%%%%%%%%% Pipeline model %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
rho_gas_p_mean=par.P_in_nom*par.M_g/(par.R*par.T_p);
V_fraction_liq_p_mean=rho_gas_p_mean*par.w_liq_in/(par.rho_liq*par.w_gas_in+rho_gas_p_mean*par.w_liq_in);

if V_fraction_liq_p_mean > 1
    V_fraction_liq_p_mean=1;
elseif V_fraction_liq_p_mean < 0
    V_fraction_liq_p_mean=0;
end
        

m_liq_p_mean=par.rho_liq*par.V_p*V_fraction_liq_p_mean;


h_mean=par.K_h*par.h_c*V_fraction_liq_p_mean; % mean height of liquid level in pipeline 
h=max(h_mean+((m_liq_p-m_liq_p_mean)/(par.A_p*(1-V_fraction_liq_p_mean)*par.rho_liq)*sin(par.incl*2*pi/360)),0);


V_gas_p=par.V_p-m_liq_p/par.rho_liq;
if V_gas_p > par.V_p
    V_gas_p=par.V_p;
elseif V_gas_p <= 0
    V_gas_p=0.0001;
end

rho_gas_p=m_gas_p/V_gas_p;
P_in=rho_gas_p*par.R*par.T_p/par.M_g;

Us_liq_in_mean=4*par.w_liq_in/(pi*par.D_p*par.D_p*par.rho_liq);
Re_p=par.rho_liq*Us_liq_in_mean*par.D_p/par.visc_liq;
frictionfactor_p=(1/(-1.8*log10((par.roughness/(par.D_p*3.7))^1.11 + 6.9/Re_p)))^2;

% Pressure change to to friction in pipeline
P_f_p=V_fraction_liq_p_mean*frictionfactor_p*par.rho_liq*Us_liq_in_mean^2*par.L_p/(2*par.D_p);


%% %%%%%%%%%%%%%%%%%%%%%%% Riser model %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

V_gas_r=par.V_r-m_liq_r/par.rho_liq;
if V_gas_r > par.V_r
    V_gas_r=par.V_r;
elseif V_gas_r < 0
    m_gas_r=0;
end

rho_gas_r=m_gas_r/V_gas_r;

% Pressure riser top
P_rt=rho_gas_r*par.R*par.T_r/par.M_g;

V_fraction_liq_r_mean=m_liq_r/(par.V_r*par.rho_liq);
rho_mix_r_mean=(m_gas_r+m_liq_r)/par.V_r;
Us_liq_r_mean=par.w_liq_in/(par.rho_liq*par.A_r);
Us_gas_r_mean=par.w_gas_in/(rho_gas_r*par.A_r);
Us_mix_r_mean=Us_liq_r_mean+Us_gas_r_mean;
Re_r=rho_mix_r_mean*Us_mix_r_mean*par.D_r/par.visc_liq;
frictionfactor_r=(1/(-1.8*log10((par.roughness/(par.D_p*3.7))^1.11 + 6.9/Re_r)))^2;

% Pressure drop due to friction in riser
P_f_r=V_fraction_liq_r_mean*frictionfactor_r*rho_mix_r_mean*Us_mix_r_mean^2*(par.L_r+par.L_h)/(2*par.D_r);


%% %%%%%%%%%%%%%%%%%%%%%%% well model %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 

V_fraction_liq_w_mean=m_liq_w/(par.V_w*par.rho_liq);

if V_fraction_liq_w_mean > 1
    V_fraction_liq_w_mean=1;
elseif V_fraction_liq_w_mean < 0
    V_fraction_liq_w_mean=0;
end

% calculating friction factor
Us_liq_w_mean=4*par.w_nom/(pi*par.D_w*par.D_w*par.rho_liq);
Re_w=par.rho_liq*Us_liq_w_mean*par.D_w/par.visc_liq;
frictionfactor_w=(1/(-1.8*log10((par.roughness/(par.D_w*3.7))^1.11 + 6.9/Re_w)))^2;

% Pressure change due to friction in well
P_f_w=V_fraction_liq_w_mean*frictionfactor_w*par.rho_liq*Us_liq_w_mean^2*par.L_w/(2*par.D_w);

rho_mix_w_mean=(m_gas_w + m_liq_w)/par.V_w;

%Pressure at well head
P_wh=m_gas_w*par.R*par.T_w/(par.M_g*(par.V_w-m_liq_w/par.rho_liq));

%Pressure at bottom hole
P_bh=P_wh+rho_mix_w_mean*par.L_w*par.g+P_f_w;

%% %%%%%%%%%%%%%%%%%%%%%%% Phase distribution at well head %%%%%%%%%%%%%%%%

V_fraction_liq_wb=1;
V_fraction_liq_wt=2*V_fraction_liq_w_mean - V_fraction_liq_wb;
if V_fraction_liq_wt > 1
    V_fraction_liq_wt=1;
elseif V_fraction_liq_wt < 0
    V_fraction_liq_wt=0;
end


rho_gas_w=m_gas_w/(par.V_w-m_liq_w/par.rho_liq);
rho_mix_wt=V_fraction_liq_wt*par.rho_liq + (1-V_fraction_liq_wt)*rho_gas_w;

m_fraction_gas_wt=(1-V_fraction_liq_wt)*rho_gas_w/rho_mix_wt;
if m_fraction_gas_wt > 1
    m_fraction_gas_wt=1;
elseif m_fraction_gas_wt < 0
    m_fraction_gas_wt=0;
end

%% %%%%%%%%%%%%%%%%%%%%%%% Flow at well head %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

w_mix_wh=par.CV_3*Z3*sqrt(rho_mix_wt*max(P_wh-P_in,0));

w_gas_wh=w_mix_wh*m_fraction_gas_wt;
w_liq_wh=w_mix_wh*(1-m_fraction_gas_wt);


%% %%%%%%%%%%%%%%%%%%%%%%% Flow at riser base %%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% Gas
P_g=P_in-P_f_p-P_rt-rho_mix_r_mean*par.g*par.L_r-P_f_r;
if h >= par.h_c
    A_g=0;
    w_gas_rb=0;
else
    A_g=par.A_p*((par.h_c-h)/par.h_c)^2;
    w_gas_rb=par.K_g*A_g*sqrt(max(rho_gas_p*P_g,0));
end

% Liquid
P_l=P_in-P_f_p+par.rho_liq*par.g*h-P_rt-rho_mix_r_mean*par.g*par.L_r-P_f_r;
A_l=par.A_p-A_g;
w_liq_rb=par.K_l*A_l*sqrt(max(par.rho_liq*P_l,0));

%% %%%%%%%%%%%%%%%%%%%%%%% Phase distribution at choke %%%%%%%%%%%%%%%%%%%% 

V_fraction_liq_rb=A_l/par.A_p;

V_fraction_liq_rt=2*m_liq_r/(par.V_r*par.rho_liq)-V_fraction_liq_rb;
if V_fraction_liq_rt > 1
    V_fraction_liq_rt=1;
elseif V_fraction_liq_rt < 0
    V_fraction_liq_rt=0;
end


%% %%%%%%%%%%%%%%%%%%%%%%% Outflow conditions %%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
rho_rt=V_fraction_liq_rt*par.rho_liq+(1-V_fraction_liq_rt)*rho_gas_r;
w_out_mix=par.CV_1*Z1*sqrt(rho_rt*max(P_rt-par.P_s,0));

%%%%%%%%%%%%%
m_fraction_liq_rt=V_fraction_liq_rt*par.rho_liq/(V_fraction_liq_rt*par.rho_liq+(1-V_fraction_liq_rt)*rho_gas_r);
if m_fraction_liq_rt > 1
    m_fraction_liq_rt=1;
elseif m_fraction_liq_rt < 0
    m_fraction_liq_rt=0;
end


%%%%%%%%%%%%%

w_liq_out=m_fraction_liq_rt*w_out_mix;
w_gas_out=(1-m_fraction_liq_rt)*w_out_mix;

%% %%%%%%%%%%%%%%%%%%%%%%% Calculate flow from reservoir %%%%%%%%%%%%%%%%%%

w_res_mix = par.PI*max(0,par.P_res-P_bh)+d_1;

w_liq_res=1/(par.gor+1)*w_res_mix;
w_gas_res=(par.gor/(par.gor+1))*w_res_mix;

%%
P_wh_noise=(P_wh+100*(rand()-0.5));

%% %%%%%%%%%%%%%%%%%%%%%%% Change in mass %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
delta_m_liq_w=w_liq_res-w_liq_wh;
delta_m_gas_w=w_gas_res-w_gas_wh;
delta_m_liq_p=w_liq_wh-w_liq_rb;
delta_m_gas_p=w_gas_wh-w_gas_rb;
delta_m_liq_r=w_liq_rb-w_liq_out;
delta_m_gas_r=w_gas_rb-w_gas_out;

dm_dt=[delta_m_liq_w; delta_m_gas_w; delta_m_liq_p; delta_m_gas_p; delta_m_liq_r; delta_m_gas_r];

%%
w_total=[w_liq_res; w_gas_res; w_liq_wh; w_gas_wh; w_liq_rb; w_gas_rb; w_liq_out; w_gas_out];
P_all.Separator=par.P_s;
P_all.riser_top=P_rt;
P_all.riser_bottom_gas=P_g;
P_all.riser_bottom_liquid=P_l;
P_all.innlet=P_in;
P_all.well_head=P_wh;
P_all.bottom_hole=P_bh;
P_all.reservoir=par.P_res;
P_all.friction_well=P_f_w;
P_all.friction_pipe=P_f_p;
P_all.friction_riser=P_f_r;

end
