%%
%This code is intial code for a well-pipeline-riser system, with a control
%system.
%
%It is based on formulas provided by Esmaeil Jahanshahi in his phd-thesis
%"Control Solutions for Multiphase Flow" from October 2013.
%
%Created by Eskil Hove Meringdal 
%
%Created during his master thesis at NTNU "Comparative study of different methods for 
%slug control" in 2014.
%
%Supervisor at NTNU: Olav Egeland
%                   
%
%

%% Description
% The setup provided uses cascade control with flow out in the inner loop,
% and pressure at the well head in the outer loop. They control the opening
% of a valve located topside.
%
% The control system is activated in a stable area with a reference
% pressure at 70 bar. After 1000 seconds it is changed to 67 bar which is
% inside the unstable area. After 4000 seconds the valve is locked in its
% current position, and the slugging starts after some time, proving that
% it is in the unstable area. It runs for 8000 seconds

%% clear exicting data and whipe screen
clear all
clc

%% Load parameters
par = parameters();

%% Number of times ode function runs
time=8000;

%% Create arrays for faster computing
Z1=ones(time,1)*0.09;
u=ones(time,1)*14.4;
Z3=ones(time,1)*1;
P_wh=zeros(time,1);
w_liq_out=zeros(time,1);
w_gas_out=zeros(time,1);
w_out_mix=zeros(time,1);
P_in=zeros(time,1);

%% Controller - inner loop(w_mix)
Kp=0.003;
Ti=5;
Td=0;
sat=0;
int=0.09;
y_b=0;

%% Controller - outer loop(P_wh)
Kp2=-0.00001;
Ti2=400;
Td2=0;
sat2=0;
int2=14.4;
y_b2=0;
P_ref=7000000;

%% Set initial values required by ode
m=par.m_init;
m_init=par.m_init;

t0=1;
tend=2;
n=100;
tspan=linspace(t0,tend,n);

options=odeset('RelTol',1e-6,'Stats','on');



%%  Run process
for k=1:time
    [t,m_ode]=ode15s(@(t, m) well_pipeline_riser_ode(t, m, par, Z1(k), Z3(k)), tspan, m_init);
    m_init=(m_ode(length(m_ode),:))';
    m=[m;m_ode];
    
    %Find variables
    [dm_dt(k,:),P_wh(k),w_out_mix(k), P_all(k,:)] = well_pipeline_riser(t,m_init,par,Z1(k),Z3(k));
    
    %Controllers
    if (k<4000)
        [u(k+1),y_b2,int2,sat2]= controller2(Kp2,Ti2,Td2,P_wh(k),int2,y_b2,P_ref,sat2);
        [Z1(k+1),y_b,int,sat]= controller(Kp,Ti,Td,w_out_mix(k),int,y_b,u(k+1),sat);
    else
        Z1(k+1)=Z1(k);
    end

    if k==1000
       P_ref=6700000;
    end
    
    %Display progress
    clc
    disp(k*100/time)
end


%% Display results
fig1=figure(1);
subplot(3,2,1);  plot(m(:,1)),title('Mass of liquid in well'), grid
subplot(3,2,2);  plot(m(:,2)),title('Mass of gas in well'), grid
subplot(3,2,3);  plot(m(:,3)),title('Mass of liquid in pipe'), grid
subplot(3,2,4);  plot(m(:,4)),title('Mass of gas in pipe'), grid
subplot(3,2,5);  plot(m(:,5)),title('Mass of liquid in riser'), grid
subplot(3,2,6);  plot(m(:,6)),title('Mass of gas in riser'), grid

fig5=figure(5);
plotyy(linspace(1,length(w_out_mix),length(w_out_mix)),w_out_mix,linspace(1,length(Z1),length(Z1)),Z1),title('Valve opening(green) and Production(blue)'), grid

display('Pressures at end of simulation[Pa]:')
display(P_all(length(P_all)))
%